"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComposeService = void 0;
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const BootstrapUtils_1 = require("./BootstrapUtils");
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
class ComposeService {
    constructor(params) {
        this.params = params;
    }
    async run() {
        const presetData = BootstrapUtils_1.BootstrapUtils.loadExistingPresetData(this.params.target);
        const workingDir = `${process.cwd()}`;
        const target = `${workingDir}/${this.params.target}`;
        const targetDocker = `${target}/docker/`;
        await BootstrapUtils_1.BootstrapUtils.mkdir(targetDocker);
        await BootstrapUtils_1.BootstrapUtils.generateConfiguration({}, `${this.params.root}/config/docker`, targetDocker);
        logger.info(`creating docker-compose.yml from last used profile.`);
        const services = {};
        (presetData.databases || []).forEach((n) => {
            const databaseService = {
                image: 'mongo',
                command: `bash -c "mongod --dbpath=/dbdata --bind_ip=${n.name}"`,
                stop_signal: 'SIGINT',
                ports: n.openPort ? ['27017:27017'] : [],
                volumes: ['../data/mongo:/dbdata:rw'],
            };
            services[n.name] = databaseService;
            services[n.name + '-init'] = {
                image: 'mongo',
                command: 'bash -c "/bin/bash /userconfig/mongors.sh && touch /state/mongo-is-setup"',
                volumes: [`./mongo:/userconfig/:ro`, '../data/mongo:/dbdata:rw', '../state:/state'],
                depends_on: [n.name],
            };
        });
        (presetData.nodes || []).forEach((n) => {
            const nodeService = {
                build: `dockerfiles/catapult`,
                command: `bash -c "/bin/bash /userconfig/runServerRecover.sh  ${n.name} && /bin/bash /userconfig/startServer.sh ${n.name}"`,
                stop_signal: 'SIGINT',
                depends_on: [],
                restart: 'on-failure:2',
                volumes: [
                    `./bin/bash:/bin-mount`,
                    `../config/${n.name}/resources/:/userconfig/resources/`,
                    '../data/' + n.name + ':/data:rw',
                    '../data/nemesis-data:/nemesis-data:rw',
                    '../state:/state',
                ],
            };
            if (n.databaseHost) {
                nodeService.depends_on.push(n.databaseHost + '-init');
            }
            services[n.name] = nodeService;
            if (n.brokerHost) {
                services[n.brokerHost] = {
                    build: `dockerfiles/catapult`,
                    command: `bash -c "/bin/bash /userconfig/runServerRecover.sh ${n.brokerHost} && /bin/bash /userconfig/startBroker.sh ${n.brokerHost}"`,
                    stop_signal: 'SIGINT',
                    ports: n.openBrokerPort ? ['7902:7902'] : [],
                    restart: 'on-failure:2',
                    volumes: [
                        `./bin/bash:/bin-mount`,
                        `../config/${n.name}/resources/:/userconfig/resources/`,
                        '../data/' + n.name + ':/data:rw',
                        '../state:/state',
                    ],
                };
                nodeService.depends_on.push(n.brokerHost);
            }
        });
        (presetData.gateways || []).forEach((n) => {
            services[n.name] = {
                image: 'symbolplatform/symbol-rest:1.2.0',
                command: 'ash -c "npm start /userconfig/rest.json"',
                stop_signal: 'SIGINT',
                ports: n.openPort ? ['3000:3000'] : [],
                volumes: [
                    `./bin/ash:/bin-mount`,
                    `../config/${n.name}/:/userconfig/`,
                    '../state:/state',
                    `../data/${n.apiNodeHost}:/logs:rw`,
                    `../config/${n.apiNodeHost}/resources/:/usr/local/share/symbol/api-node-config/`,
                ],
                depends_on: [n.databaseHost + '-init'],
                networks: {
                    default: {
                        ipv4_address: '172.20.0.10',
                    },
                },
            };
        });
        const dockerCompose = {
            version: '3',
            networks: {
                default: {
                    ipam: {
                        config: [
                            {
                                subnet: '172.20.0.0/24',
                            },
                        ],
                    },
                },
            },
            services: services,
        };
        const dockerFile = `${target}/docker/docker-compose.yml`;
        await BootstrapUtils_1.BootstrapUtils.writeYaml(dockerFile, dockerCompose);
        logger.info(`docker-compose.yml file created ${dockerFile}`);
    }
}
exports.ComposeService = ComposeService;
