"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigService = exports.NodeType = void 0;
// eslint-disable-next-line @typescript-eslint/no-var-requires
const BootstrapUtils_1 = require("./BootstrapUtils");
const symbol_sdk_1 = require("symbol-sdk");
const CertificateService_1 = require("./CertificateService");
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const NemgenService_1 = require("./NemgenService");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const fs = require('fs');
var NodeType;
(function (NodeType) {
    NodeType["PEER_NODE"] = "peer-node";
    NodeType["API_NODE"] = "api-node";
})(NodeType = exports.NodeType || (exports.NodeType = {}));
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
class ConfigService {
    constructor(params = ConfigService.defaultParams) {
        this.params = params;
    }
    toConfig(account) {
        return {
            privateKey: account.privateKey,
            publicKey: account.publicAccount.publicKey,
            address: account.address.plain(),
        };
    }
    static getNetworkIdentifier(networkType) {
        switch (networkType) {
            case symbol_sdk_1.NetworkType.MAIN_NET:
                return 'public';
            case symbol_sdk_1.NetworkType.TEST_NET:
                return 'public-test';
            case symbol_sdk_1.NetworkType.MIJIN:
                return 'mijin';
            case symbol_sdk_1.NetworkType.MIJIN_TEST:
                return 'mijin-test';
        }
        throw new Error(`Invalid Network Type ${networkType}`);
    }
    static getNetworkName(networkType) {
        switch (networkType) {
            case symbol_sdk_1.NetworkType.MAIN_NET:
                return 'public';
            case symbol_sdk_1.NetworkType.TEST_NET:
                return 'publicTest';
            case symbol_sdk_1.NetworkType.MIJIN:
                return 'mijin';
            case symbol_sdk_1.NetworkType.MIJIN_TEST:
                return 'mijinTest';
        }
        throw new Error(`Invalid Network Type ${networkType}`);
    }
    generateAddresses(networkType, size) {
        return ConfigService.getArray(size).map((_) => this.toConfig(symbol_sdk_1.Account.generateNewAccount(networkType)));
    }
    async generateNodeAccount(index, node, networkType) {
        const type = node.type;
        const name = node.name || `${type}-${index}`;
        const signing = this.toConfig(symbol_sdk_1.Account.generateNewAccount(networkType));
        const vrf = this.toConfig(symbol_sdk_1.Account.generateNewAccount(networkType));
        const ssl = await new CertificateService_1.CertificateService(this.params).run(name);
        return BootstrapUtils_1.BootstrapUtils.removeUndefined({ signing, vrf, ssl, type, name });
    }
    async generateNodeAccounts(networkType, nodes) {
        return Promise.all(nodes.map((node, index) => {
            return this.generateNodeAccount(index, node, networkType);
        }));
    }
    generateAddress(networkType) {
        return this.toConfig(symbol_sdk_1.Account.generateNewAccount(networkType));
    }
    static getArray(size) {
        return [...Array(size).keys()];
    }
    async run() {
        const presetData = BootstrapUtils_1.BootstrapUtils.loadPresetData(this.params.root, this.params.preset, this.params.presetFile);
        const configFolder = `${this.params.target}/config`;
        if (this.params.reset) {
            logger.info('Deleting config folder');
            BootstrapUtils_1.BootstrapUtils.deleteFolderRecursive(configFolder);
        }
        if (fs.existsSync(configFolder)) {
            logger.info('Config folder exist, ignoring configuration. (run -r to reset)');
            return;
        }
        logger.info(`Generating config from preset ${this.params.preset} ` + (this.params.presetFile || ''));
        await fs.promises.mkdir(`${this.params.target}/config/generated-addresses`, { recursive: true });
        const networkType = presetData.networkType;
        const addresses = await this.generateRandomConfiguration(networkType, presetData);
        this.completePresetDataWithRandomConfiguration(presetData, addresses, networkType);
        await BootstrapUtils_1.BootstrapUtils.writeYaml(`${this.params.target}/config/generated-addresses/addresses.yml`, addresses);
        await this.generateNodes(presetData, addresses);
        await this.generategateways(presetData, addresses);
        await BootstrapUtils_1.BootstrapUtils.mkdir(`${this.params.target}/data/nemesis-data/seed/00000`);
        if (presetData.nemgen) {
            await this.generateNemesisConfig(presetData, addresses);
        }
        else {
            const copyFrom = `${this.params.root}/presets/${this.params.preset}/seed`;
            const copyTo = `${this.params.target}/data/nemesis-data/seed`;
            await BootstrapUtils_1.BootstrapUtils.generateConfiguration({}, copyFrom, copyTo);
        }
        await BootstrapUtils_1.BootstrapUtils.writeYaml(`${this.params.target}/config/preset.yml`, presetData);
        logger.info(`Configuration generated.`);
    }
    completePresetDataWithRandomConfiguration(presetData, addresses, networkType) {
        presetData.networkIdentifier = ConfigService.getNetworkIdentifier(presetData.networkType);
        presetData.networkName = ConfigService.getNetworkName(presetData.networkType);
        if (!presetData.nemesisGenerationHashSeed) {
            presetData.nemesisGenerationHashSeed = addresses.nemesisGenerationHashSeed;
        }
        //How can it work?
        const ownerAddress = symbol_sdk_1.Address.createFromPublicKey(presetData.nemesisSignerPublicKey, networkType);
        if (!presetData.currencyMosaicId)
            presetData.currencyMosaicId = BootstrapUtils_1.BootstrapUtils.toHex(symbol_sdk_1.MosaicId.createFromNonce(symbol_sdk_1.MosaicNonce.createFromNumber(0), ownerAddress).toHex());
        if (!presetData.harvestingMosaicId) {
            if (presetData.mosaics && presetData.mosaics.length > 1) {
                presetData.harvestingMosaicId = BootstrapUtils_1.BootstrapUtils.toHex(symbol_sdk_1.MosaicId.createFromNonce(symbol_sdk_1.MosaicNonce.createFromNumber(1), ownerAddress).toHex());
            }
            else {
                presetData.harvestingMosaicId = presetData.currencyMosaicId;
            }
        }
    }
    async generateRandomConfiguration(networkType, presetData) {
        const addresses = {
            networkType: networkType,
            nemesisGenerationHashSeed: presetData.nemesisGenerationHashSeed || symbol_sdk_1.Account.generateNewAccount(networkType).publicAccount.publicKey,
        };
        if (presetData.nodes) {
            addresses.nodes = await this.generateNodeAccounts(networkType, presetData.nodes);
        }
        if (presetData.gateways) {
            addresses.gateways = this.generateAddresses(networkType, presetData.gateways.length);
        }
        if (presetData.nemgen) {
            addresses.nemesisSigner = this.generateAddress(networkType);
        }
        if (!presetData.nemesisSignerPrivateKey && addresses.nemesisSigner) {
            presetData.nemesisSignerPrivateKey = addresses.nemesisSigner.privateKey;
        }
        if (!presetData.nemesisSignerPublicKey && addresses.nemesisSigner) {
            presetData.nemesisSignerPublicKey = addresses.nemesisSigner.publicKey;
        }
        if (presetData.mosaics) {
            const mosaics = {};
            presetData.mosaics.forEach((m) => {
                mosaics[m.name] = this.generateAddresses(networkType, m.accounts);
            });
            presetData.mosaics.forEach((m) => {
                const accounts = mosaics[m.name];
                if (!m.currencyDistributions) {
                    m.currencyDistributions = accounts.map((a) => {
                        return { address: a.address, amount: m.supply / m.accounts };
                    });
                }
            });
            addresses.mosaics = mosaics;
        }
        return addresses;
    }
    async generateNodes(presetData, addresses) {
        await Promise.all((addresses.nodes || []).map(async (account, index) => {
            await this.generateNodeConfiguration(account, index, presetData, addresses);
        }));
    }
    async generateNodeConfiguration(account, index, presetData, addresses) {
        const copyFrom = `${this.params.root}/config/node`;
        const name = account.name;
        const outputFolder = `${this.params.target}/config/${name}`;
        const generatedContext = {
            name: name,
            harvesterSigningPrivateKey: account.signing.privateKey,
            harvesterVrfPrivateKey: account.vrf.privateKey,
        };
        const templateContext = Object.assign(Object.assign(Object.assign({}, presetData), generatedContext), (presetData.nodes || [])[index]);
        await BootstrapUtils_1.BootstrapUtils.generateConfiguration(templateContext, copyFrom, outputFolder);
        await this.generateP2PFile(presetData, addresses, outputFolder, NodeType.PEER_NODE, 'peers-p2p.json');
        await this.generateP2PFile(presetData, addresses, outputFolder, NodeType.API_NODE, 'peers-api.json');
    }
    async generateP2PFile(presetData, addresses, outputFolder, type, jsonFileName) {
        var _a;
        const thisNetworkKnownPeers = (addresses.nodes || [])
            .map((node, index) => {
            if (node.type !== type) {
                return undefined;
            }
            const nodePresetData = (presetData.nodes || [])[index];
            const name = node.name;
            return {
                publicKey: node.ssl.publicKey,
                endpoint: {
                    host: name,
                    port: 7900,
                },
                metadata: {
                    name: name,
                    roles: nodePresetData.roles,
                },
            };
        })
            .filter((i) => i);
        const globalKnownPeers = ((_a = presetData.knownPeers) === null || _a === void 0 ? void 0 : _a[type]) || [];
        const data = {
            _info: `this file contains a list of ${type} peers`,
            knownPeers: [...thisNetworkKnownPeers, ...globalKnownPeers],
        };
        await fs.promises.writeFile(outputFolder + '/resources/' + jsonFileName, JSON.stringify(data, null, 2));
    }
    async generateNemesisConfig(presetData, addresses) {
        await fs.promises.mkdir(`${this.params.target}/config${presetData.transactionsDirectory}`, { recursive: true });
        const copyFrom = `${this.params.root}/config/nemesis`;
        const moveTo = `${this.params.target}/config/nemesis`;
        const templateContext = Object.assign(Object.assign({}, presetData), { addresses });
        await Promise.all((addresses.nodes || []).map((n) => this.createVrfTransaction(presetData, n)));
        await BootstrapUtils_1.BootstrapUtils.generateConfiguration(templateContext, copyFrom, moveTo);
        await new NemgenService_1.NemgenService(this.params).run(presetData.networkIdentifier);
    }
    async createVrfTransaction(presetData, node) {
        const transactionsDirectory = `${this.params.target}/config${presetData.transactionsDirectory}`;
        const deadline = symbol_sdk_1.Deadline['createFromDTO']('1');
        const vrf = symbol_sdk_1.VrfKeyLinkTransaction.create(deadline, node.vrf.publicKey, symbol_sdk_1.LinkAction.Link, presetData.networkType, symbol_sdk_1.UInt64.fromUint(0));
        const account = symbol_sdk_1.Account.createFromPrivateKey(node.signing.privateKey, presetData.networkType);
        const signedTransaction = account.sign(vrf, presetData.nemesisGenerationHashSeed);
        await fs.promises.writeFile(`${transactionsDirectory}/vrf_${node.name}.bin`, symbol_sdk_1.Convert.hexToUint8(signedTransaction.payload));
    }
    generategateways(presetData, addresses) {
        return Promise.all((addresses.gateways || []).map((account, index) => {
            const copyFrom = `${this.params.root}/config/rest-gateway`;
            const generatedContext = {
                restPrivateKey: account.privateKey,
            };
            const gatewayPreset = (presetData.gateways || [])[index];
            const templateContext = Object.assign(Object.assign(Object.assign({}, presetData), generatedContext), gatewayPreset);
            const name = templateContext.name || `rest-gateway-${index}`;
            const moveTo = `${this.params.target}/config/${name}`;
            return BootstrapUtils_1.BootstrapUtils.generateConfiguration(templateContext, copyFrom, moveTo);
        }));
    }
}
exports.ConfigService = ConfigService;
ConfigService.defaultParams = { target: 'target', preset: 'bootstrap', reset: false, root: '.' };
